from copy import deepcopy


class NQueensCSP:
    
    def __init__(self, n):
        self.n = n
        
    def is_valid(self, assignment):
        for X, u in enumerate(assignment[:-1]):
            if u is None:
                continue
            for Y, v in enumerate(assignment[X+1:], start=X+1):
                if v is None:
                    continue
                if u == v or abs(X - Y) == abs(u - v):
                    return False
        return True
    
    def is_complete(self, assignment):
        return None not in assignment
    
    def print(self, assignment):
        mat = [['_' for j in range(self.n)] for i in range(self.n)]
        for X, v in enumerate(assignment):
            if v is not None:
                mat[v][X] = 'Q'
        for row in mat:
            print('|'.join(row))
        print()
    
    def forward_check(self, assignment, domains):
        new_domains = deepcopy(domains)
        for X, D in enumerate(new_domains):
            if assignment[X] is not None:
                continue
            for Y, v in enumerate(assignment):
                if v is None or X == Y:
                    continue
                D.discard(v)
                D.discard(v - abs(X - Y))
                D.discard(v + abs(X - Y))     
        return new_domains
    
    def select_variable(self, assignment, domains):
        best_X = None
        best_len_D = float('+inf')
        for X, D in enumerate(domains):
            if assignment[X] is None:
                if len(D) < best_len_D:
                    best_X = X
                    best_len_D = len(D)
        return best_X